#if defined _wS_vip_included
	#endinput
#endif
#define _wS_vip_included

// https://world-source.ru/sm_lib/scripter_functions.zip
#include <scripter_functions>

// Инструкция по созданию модулей:
// https://world-source.ru/forum/141-4848-1

#define STEAMID_LENGTH				32
#define GROUP_LENGTH				40
#define ITEM_NAME_LENGTH			40	// item_name  'speed'
#define ITEM_INFO_LENGTH			100	// item_value '0.5'
#define DESCRIPTION_LENGTH			320 // public bool VIP_Description

#define PLAY_SELECT_SOUND(%0)		ClientCommand(%0, "playgamesound buttons/button14.wav")
#define PLAY_EXIT_SOUND(%0)			ClientCommand(%0, "playgamesound buttons/combine_button7.wav")
#define PLAY_EXIT_BACK_SOUND(%0)	ClientCommand(%0, "playgamesound buttons/combine_button7.wav")
#define PLAY_WARNING_SOUND(%0)		ClientCommand(%0, "playgamesound resource/warning.wav")

//

// Используется в 'public VipGiveAction VIP_GiveAccess'.
#define CURRENT_ACCESS_NO			0 // У игрока нет фактического доступа к функции (возможно это авто-выдача доступа по флагу -> settings.txt -> "_ACCESS_BY_FLAG").
#define CURRENT_ACCESS_YES			1 // Фактический доступ есть.
#define CURRENT_ACCESS_YES_GROUP	2 // Фактический доступ есть + это VIP игрок и у его VIP группы тоже есть доступ.

enum VipGiveAction
{
	VGA_Continue = 0,		// Разрешить/продолжить нормальное выполнение.
	VGA_Stop,				// Запретить (без причины).
	VGA_ValueNotSpecified,	// Запретить (значение не указано).
	VGA_BadValue,			// Запретить (значение указано неверно).
	VGA_SameValue,			// Запретить (значение равно тому, что сейчас у игрока).
	VGA_AlreadyEnabled,		// Запретить (функция уже включена).
	VGA_Paused				// Запретить (игрок отключил функцию и мы не хотим принудительно её включать) - в некоторых случаях, чтобы это обойти, нужно указать любое [item_value].
};

//

enum VIP_EVENT
{
	// Авторизован.
	// public void Callback(int client, const char[] steamid, bool vip);
	VE_Authorized = 0,
	
	// Рождён.
	// public void Callback(int client, int team);
	VE_Spawn,
	
	// Получил VIP доступ .
	// public void Callback(int client, const char[] steamid, const char[] group, int min, const char[] admin);
	VE_Added,				
	
	// Лишился VIP доступа.
	// public void Callback(int client, const char[] steamid, const char[] reason);
	// @param reason = "time is up", "server", "wS, STEAM_0:0:xxxxxxx".
	VE_Removed,
	
	// VIP группа изменена.
	// public void Callback(int client, const char[] steamid, const char[] oldGroup, const char[] newGroup);
	VE_GroupChanged,
	
	// Активировал ключ.
	// public void Callback(int client, const char[] steamid, const char[] key, const char[] group, int min, const char[] cmd, bool timeChanged, bool groupChanged);
	VE_KeyActivated,
	
	// null = соединение разорвано, иначе восстановлено.
	// public void Callback(Database database);
	VE_Sql,
	
	// Бесплатный VIP в определённое время суток. @param started true = начался, false = завершился.
	// public void Callback(bool started);
	VE_FreeTime,
	
	// Получает бесплатный VIP. Plugin_Stop или Plugin_Handled = запретить. Plugin_Continue = разрешить.
	// public Action Callback(int client);
	VE_FreeGive,
	
	// Берёт отпуск. Если min < 1, то изменится на 1.
	// public Action Callback(int client, int &min);
	VE_HolidayPre,
	
	// Взял отпуск.
	// public void Callback(int client, int min);
	VE_HolidayPost,
	
	// Отпуск завершён/отменён. @param sec = сколько секунд длился.
	// public void Callback(int client, int sec);
	VE_HolidayEnd,
	
	// enabled true = функция включена/изменена (выбрал другой скин например), false = отключена. item_value = установленное значение (TrimString уже сделано).
	VE_ItemStatusChanged,	// public void Callback(int client, const char[] item_name, bool enabled, const char[] item_value); - 
	VE_ModuleLoaded,		// public bool Callback(int item_id, const char[] item_name);	VIP зарегистрировал модуль (VIP_RegisterItem). return false = принудительно ставится флаг VIP_ITEM_MAPDISABLED). return true = если нет флага VIP_ITEM_MAPDISABLED, то функция сразу же включается у всех игроков, у которых есть доступ.
	VE_ModuleUnloaded		// public void Callback(int item_id, const char[] item_name);	Модуль выгружен (VIP_UnRegisterItem или unload/краш).
};

native void VIP_HookEvent(VIP_EVENT event, Function callback);

//

/**
 * Если функция была отключена/включена/изменена, то можно (в конце, после всех действий) отправить уведомление для VE_ItemStatusChanged.
 * item_value полезен при bool enabled true, чтобы могли узнать установленное игроку значение.
 * Например, админ сменил игроку скорость на 0.3, вот в item_value мы и передадим "0.3".
 * После отправки уведомления, VE_ItemStatusChanged будет вызван через 0.1 сек.
 **/
native void VIP_NotifyItemStatusChanged(int client, const char[] item_name, bool enabled, const char[] item_value="", int item_value_length=0);

//

#define VCF_AUTHORIZED			(1 << 0)	// Проверки доступа завершены (авторизован) 'if (VIP_GetFlags(client) & VCF_AUTHORIZED)'
#define VCF_VIP					(1 << 1)	// VIP игрок находится в SQL базе или он VCF_VIPFREE/VCF_VIPTEMPORARY.
#define VCF_VIPFREE				(1 << 2)	// VIP права выданы функцией 'бесплатный VIP в определенное время' (VE_FreeTime, VE_FreeGive) (вместе с этим флагом игроку ставится и VCF_VIPTEMPORARY)
#define VCF_VIPTEMPORARY		(1 << 3)	// VIP права выданы функцией VIP_SetTemporaryGroup (возможно игрок прописан в vips.txt)
#define VCF_VIPPAUSE			(1 << 4)	// Игрок воспользовался функцией "Отключить всё (перезаход = вкл)", при этом у него забирается флаг VCF_AUTHORIZED.
#define VCF_CHATHOOK			(1 << 5)	// Игрок (скорее всего админ) должен ввести какое-то значение в чат и сообщение будет заблокировано.
#define VCF_MENUAUTOOPEN		(1 << 6)	// Меню отправляется игроку снова и снова (TIMER_REPEAT), пока он его не покинет.
#define VCF_SAVECACHE			(1 << 7)	// Личные настройки игрока (cfg/vip/cache/) должны быть сохранены при выходе.
#define VCF_HOLIDAY				(1 << 8)	// В отпуске (при этом у него есть флаг VCF_AUTHORIZED).
#define VCF_VIPTEST				(1 << 9)	// VIP права получены через функцию sm_viptest
#define VCF_VIPBYFLAG			(1 << 10)	// Есть функции, доступ к которым был получен по флагу.
#define VCF_VIPMENUACCESS		(1 << 11)	// Есть ли доступ к какой-либо функции. Если есть, то она доступна игроку в !vip меню (неважно VIP он или нет).

native int VIP_GetFlags(int client);

//

#define VIP_ITEM_TOGGLE			(1 << 0)	// Игрок может вкл/откл функцию. Вызываются VIP_GiveAccess/VIP_TakeAccess
#define VIP_ITEM_SELECT			(1 << 1)	// При нажатии вызывается VIP_MenuSelect (полезно, если нужно показать игроку своё меню)
#define VIP_ITEM_DISABLED		(1 << 2)	// Это ITEMDRAW_DISABLED - белая некликабельная опция
#define VIP_ITEM_MAPDISABLED	(1 << 3)	// Функция отключена на текущей карте (но админ может включить её любому игроку)
#define VIP_ITEM_HIDDEN			(1 << 4)	// Опция скрыта (не отображается в !vip меню)

native int VIP_GetItemFlags(int item_id);

/**
 * Добавить функцию в !vip меню.
 *
 * Если указать VIP_ITEM_TOGGLE:
 * 		Функция будет вкл/откл на первой странице !vip меню или на странице с описанием функции.
 * 		Описание добавляется с помощью public bool VIP_Description и vip_modules_description.phrases.txt
 * 		Когда игрок включает функцию, то вызывается VIP_GiveAccess и в кэш записывается "item_name" "1".
 * 		Отключает: вызывается VIP_TakeAccess и "item_name" удаляется из кэша.
 * 
 * Возвращает уникальный item_id от 1 и выше (0 = ошибка).
**/
native int VIP_RegisterItem(const char[] item_name, int item_flags);

/**
 * Удалить функцию из !vip меню.
 *
 * Для каждого VIP_RegisterItem можно сделать VIP_UnRegisterItem.
 * Но VIP теперь видит выгрузку/перезагрузку/краш плагинов и он сам всё удалит.
 * Это может быть полезно только если нужно удалить функцию во время работы плагина, а не перед его отключением.
**/
native void VIP_UnRegisterItem(const char[] item_name);				

/**
 * Плагин, сделавший VIP_RegisterItem, может использовать необязательные public функции ниже.
 * Более подробно: https://world-source.ru/forum/141-4848-1
 * 
 * public VipGiveAction VIP_GiveAccess(int client, int item_id, char item_value[ITEM_INFO_LENGTH], bool callAfterAdminAction, int currentAccess, KeyValues kv)
 * {
 * }
 * 
 * public void VIP_TakeAccess(int client, int item_id, bool toggledByClient)
 * {
 *  Игрок лишился доступа к функции.
 *  Если @param toggledByClient true, то игрок просто сам её временно отключил через !vip меню.
 * }
 * 
 * public void VIP_MenuSelect(int client, int item_id)
 * {
 *  Вызывается при выборе VIP_ITEM_SELECT функции в !vip меню.
 * }
 * 
 * public bool VIP_CurrentItemValue(int client, int item_id, char[] item_value, int item_value_length)
 * {
	Смотрите   VIP_GetCurrentItemValue.
 * }
 * 
 * public bool VIP_Description(int client, int item_id, char item_value[ITEM_INFO_LENGTH], char[] description, int description_size)
 * {
 *	Это только для VIP_ITEM_TOGGLE функций.
 *
 *  return true		Откроется страница с @param description описанием функции. Вкл/откл функции происходит на этой странице.
 *					Может быть полезно, чтобы более подробно описать, как работает функция.
 *					Если используете перевод, то добавьте его в vip_modules_description.phrases.
 *
 *  return false	Не показывать страницу с описанием, а вкл/откл функцию и открыть ту же страницу.
 * }
**/

native int VIP_GetServerType();							// 1  = csgo, 2 = css ob, 3 = css v34.
native int VIP_GetSeconds(int client);					// -2 = нет VIP прав, -1 = вечный VIP доступ, x = оставшиеся секунды VIP доступа.
native void VIP_ShowMenu(int client, bool delay=false);	// Если delay true, то меню будет показано через 0.1 сек.
native int VIP_GetItemID(const char[] item_name);		// Получить item_id функции по её имени.

/**
 * Можно использовать на любом игроке (неважно есть у него VIP или нет), т.к. функция может быть выдана любому не-бот игроку.
 * Например, игроку добавлена скорость 0.5. В группе это прописывается так: speed : 0.5
 * Чтобы узнать текущее значение скорости игрока (а оно может меняться в любой момент админом/sm_vipsetitemstatus/сам включил/отключил), то делаем так:
 * 
 * char item_value[ITEM_INFO_LENGTH];
 * if (VIP_GetCurrentItemValue(client, "speed", item_value, sizeof(item_value))) {
 *     PrintToServer("Текущая скорость: '%s'", item_value);
 * }
 * else{
 *     PrintToServer("Или модуль не реализовал функцию VIP_CurrentItemValue, или вернул false.");
 * }
 * 
 * Чтобы это сработало, модуль должен добавить публичную функцию и реализовать её:
 * public bool VIP_CurrentItemValue(int client, int item_id, char[] item_value, int item_value_length);
 * Смотрите модуль speed или инструкцию (https://world-source.ru/forum/141-4848-1) чтобы узнать больше.
 * 
 * Ещё текущее значение можно узнать с помощью VE_ItemStatusChanged, но только если модуль реализовал 'VIP_NotifyItemStatusChanged'.
**/
native bool VIP_GetCurrentItemValue(int client, const char[] item_name, char[] item_value, int item_value_length);

native bool VIP_GetItemName (int item_id, char[] buffer, int maxlength=ITEM_NAME_LENGTH); // Получить имя функции по её id.
native bool VIP_GetGroupName(int client, char[] buffer, int maxlength=GROUP_LENGTH); // В какой VIP группе игрок. false = нет VIP доступа.
native bool VIP_GetGroupItemValue(const char[] group, const char[] item_name, char[] buffer, int maxlength=ITEM_INFO_LENGTH); // true, если у группы есть доступ к item_name.
native bool VIP_IsValidGroup(const char[] group); // Существует ли группа.
native bool VIP_IsVipFreeTime(); // true, если сейчас выдаётся бесплатный VIP в определённое время суток.

/**
 * Возвращает timestamp, когда игрок брал отпуск в последний раз.
 * 0 = еще не брал отпуск
**/
native int VIP_GetLastHolidayTime(int client);

/**
 * Возвращает личные настройки игрока (перед возвращением делается KvRewind).
 * Можно использовать на ЛЮБЫХ игроках.
 * Даже если у игрока нет KV файла, это вернёт пустую KeyValues группу "settings" {}
 * null = игрок не авторизован или другая ошибка.
**/
native KeyValues VIP_GetClientCache(int client);

/**
 * Сохранить личные настройки игрока в cfg/vip/cache.
 * Можно использовать на ЛЮБЫХ игроках.
 * Пригодится, если вы делали VIP_GetClientCache и что-то изменили в KV, или просто не хотите ждать, пока игрок выйдет с сервера.
 * Старайтесь сохранять KV лишь тех игроков, у которых есть доступ. Чем меньше файлов, тем лучше.
 * immediately - если true, то прямо сейчас, false - в момент выхода с сервера.
**/
native void VIP_SaveClientCache(int client, bool immediately);

/**
 * Есть ли у игрока (неважно VIP он или нет) доступ к функции.
 * Доступ к функциям может в любой момент меняться через управление модулями в админке или VIP_SetItemStatus/sm_vipsetitemstatus.
 * Чтобы узнать выдан ли этот доступ по флагу:
 * bool accessByFlag;
 * if (VIP_HaveAccessToItem(client, item_id, bAccessByFlag)) {
 *     PrintToServer("Доступ есть (по флагу: %s)", accessByFlag ? "да" : "нет");
 * }
**/
native bool VIP_HaveAccessToItem(int client, int item_id, bool &accessByFlag=false);

/**
 * Возвращает количество функций, доступных игроку (размер ar), или 0, если нет доступа ни к одной функции.
 * 
 * ArrayList ar;
 * int index = VIP_GetClientItemList(client, ar);
 * if (index > 0)
 * {
 *		int info[2];
 *		char item_name[ITEM_NAME_LENGTH];
 *		while (--index > -1)
 *		{
 *			ar.GetArray(index, info, sizeof(info));
 *			item_name[0] = 0;
 *			VIP_GetItemName(info[0], item_name, sizeof(item_name));
 *			PrintToChat(client, "Есть доступ к item_id %d (%s) (по флагу: %s)", info[0], item_name, info[1] ? "да" : "нет");
 *		}
 *		CloseHandle(ar); // Не забудьте закрыть.
 * }
**/
native int VIP_GetClientItemList(int client, ArrayList &ar);

/**
 * Это работает так же, как и RCON команда:
 * sm_vipsetitemstatus <item_name/all> <1=on|0=off> [UserID] [item_value]
 * Если item_id < 1, то это "all", т.е. действие на всех функциях, а не на одной.
**/
native void VIP_SetItemStatus(int item_id, bool enabled, int client=0, const char[] item_value="");

/**
 * Если нет связи с MYSQL/SQLite, вернёт null.
 * Таблицы/поля есть в VIP теме (https://world-source.ru/forum/134-2543-1).
**/
native Database VIP_GetSqlHandle();

/**
 * ArrayList ar = null;
 * VIP_GetItemList(ar);
 * if (ar)
 * {
 *		int index = ar.Length;
 * 		if (index > 0)
 * 		{
 *			char item_name[ITEM_NAME_LENGTH];
 *			while (--index > -1)
 *			{
 *				ar.GetString(index, item_name, sizeof(item_name));
 *				PrintToServer("%s", item_name);
 *			}
 *		}
 *		CloseHandle(ar); // Не забудьте закрыть.
 * }
**/
native void VIP_GetItemList (ArrayList &ar); // Список созданных модулями функций.
native void VIP_GetGroupList(ArrayList &ar); // Список созданных групп.

/**
 * Если в вашем плагине есть меню, можно следить чтобы игрок не слишком часто его использовал.
 * Время настраивается в vip.cfg (vip_menu_flood "0.1")
**/
native bool VIP_MenuFlood(int client, bool warningSound=true);

/**
 * Можно использовать лишь на авторизованных не-VIP игроках.
 * В SQL базу игрок не добавляется.
 * Устанавливает игроку указанную VIP группу и игрок становится VIP игроком.
 * Время устанавливается в 0 (доступ навсегда). После выхода с сервера, доступ пропадает.
 * Если игрок не был в VIP группе, доступ будет выдан, сработает VIP_EVENT_ADDED и функция вернёт true.
 * Если игрок уже во временной VIP группе и group с ней не совпадает, доступ будет изменён, сработает VIP_EVENT_GROUPCHANGED и функция вернёт true.
**/
native bool VIP_SetTemporaryGroup(int client, const char[] group);

/**
 * Отнять VIP доступ, который был выдан с помощью VIP_SetTemporaryGroup.
 * Если вернёт true, то доступ был отобран и сработал VIP_EVENT_REMOVED с reason "VIP_UnSetTemporaryGroup".
 * Не нужно использовать в OnClientDisconnect, т.к. это сделает сам VIP плагин (VIP_EVENT_REMOVED вызван не будет).
**/
native bool VIP_UnSetTemporaryGroup(int client);

public SharedPlugin __pl_vip = 
{
	name = "vip",
	file = "vip.smx",
#if defined REQUIRE_PLUGIN
	required = 1,
#else
	required = 0,
#endif
};

#if !defined REQUIRE_PLUGIN
public void __pl_vip_SetNTVOptional()
{
	MarkNativeAsOptional("VIP_HookEvent");
	MarkNativeAsOptional("VIP_NotifyItemStatusChanged");
	MarkNativeAsOptional("VIP_GetCurrentItemValue");
	MarkNativeAsOptional("VIP_GetFlags");
	MarkNativeAsOptional("VIP_GetItemFlags");
	MarkNativeAsOptional("VIP_RegisterItem");
	MarkNativeAsOptional("VIP_UnRegisterItem");
	MarkNativeAsOptional("VIP_GetServerType");
	MarkNativeAsOptional("VIP_GetSeconds");
	MarkNativeAsOptional("VIP_ShowMenu");
	MarkNativeAsOptional("VIP_GetItemID");
	MarkNativeAsOptional("VIP_GetItemName");
	MarkNativeAsOptional("VIP_GetGroupName");
	MarkNativeAsOptional("VIP_GetGroupItemValue");
	MarkNativeAsOptional("VIP_IsValidGroup");
	MarkNativeAsOptional("VIP_IsVipFreeTime");
	MarkNativeAsOptional("VIP_GetClientCache");
	MarkNativeAsOptional("VIP_SaveClientCache");
	MarkNativeAsOptional("VIP_HaveAccessToItem");
	MarkNativeAsOptional("VIP_GetClientItemList");
	MarkNativeAsOptional("VIP_SetItemStatus");
	MarkNativeAsOptional("VIP_GetSqlHandle");
	MarkNativeAsOptional("VIP_GetItemList");
	MarkNativeAsOptional("VIP_GetGroupList");
	MarkNativeAsOptional("VIP_MenuFlood");
	MarkNativeAsOptional("VIP_SetTemporaryGroup");
	MarkNativeAsOptional("VIP_UnSetTemporaryGroup");
	MarkNativeAsOptional("VIP_GetLastHolidayTime");
}
#endif